# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Classes

#### Public Classes

* [`pulpcore`](#pulpcore): Manage your next generation Pulp server
* [`pulpcore::plugin::certguard`](#pulpcoreplugincertguard): Pulp Certguard plugin
* [`pulpcore::plugin::container`](#pulpcoreplugincontainer): Pulp Container plugin
* [`pulpcore::plugin::deb`](#pulpcoreplugindeb): Pulp Deb plugin
* [`pulpcore::plugin::file`](#pulpcorepluginfile): Pulp File plugin
* [`pulpcore::plugin::migration`](#pulpcorepluginmigration): Pulp 2 to Pulp 3 Migration Plugin
* [`pulpcore::plugin::rpm`](#pulpcorepluginrpm): Pulp RPM plugin

#### Private Classes

* `pulpcore::apache`: Configure an Apache vhost
* `pulpcore::config`: Configures pulp3
* `pulpcore::database`: Set up the PostgreSQL and Redis databases
* `pulpcore::install`: Install pulpcore packages, configure user and group
* `pulpcore::service`: configure, enable, and start pulpcore services

### Defined types

* [`pulpcore::admin`](#pulpcoreadmin): Run a pulpcore-manager command
* [`pulpcore::plugin`](#pulpcoreplugin): Install a plugin

## Classes

### `pulpcore`

Manage your next generation Pulp server

#### Examples

##### 

```puppet
include pulpcore
```

#### Parameters

The following parameters are available in the `pulpcore` class.

##### `cache_dir`

Data type: `Stdlib::Absolutepath`

Pulp cache directory

Default value: `'/var/lib/pulp/tmp'`

##### `config_dir`

Data type: `Stdlib::Absolutepath`

Pulp configuration directory

Default value: `'/etc/pulp'`

##### `user`

Data type: `String`

Pulp user

Default value: `'pulp'`

##### `group`

Data type: `String`

Pulp user group

Default value: `'pulp'`

##### `user_home`

Data type: `Stdlib::Absolutepath`

Pulp user home directory

Default value: `'/var/lib/pulp'`

##### `manage_apache`

Data type: `Boolean`

Deploy a separate apache vhost for pulp3

Default value: ``true``

##### `api_host`

Data type: `Stdlib::Host`

API service host

Default value: `'127.0.0.1'`

##### `api_port`

Data type: `Stdlib::Port`

API service port

Default value: `24817`

##### `content_host`

Data type: `Stdlib::Host`

Content service host

Default value: `'127.0.0.1'`

##### `content_port`

Data type: `Stdlib::Port`

Content service port

Default value: `24816`

##### `webserver_static_dir`

Data type: `Stdlib::Absolutepath`

Directory for Pulp webserver static content

Default value: `'/var/lib/pulp/docroot'`

##### `pulp_static_root`

Data type: `Stdlib::Absolutepath`

Root directory for collected static content

Default value: `'/var/lib/pulp/assets'`

##### `postgresql_db_name`

Data type: `String`

Name of Pulp PostgreSQL database

Default value: `'pulpcore'`

##### `postgresql_db_user`

Data type: `String`

Pulp PostgreSQL database user

Default value: `'pulp'`

##### `postgresql_db_password`

Data type: `String`

Password of Pulp PostgreSQL database

Default value: `extlib::cache_data('pulpcore_cache_data', 'db_password', extlib::random_password(32))`

##### `postgresql_db_host`

Data type: `Stdlib::Host`

Host to connect to Pulp PostgreSQL database

Default value: `'localhost'`

##### `postgresql_db_port`

Data type: `Stdlib::Port`

Port to connect to Pulp PostgreSQL database

Default value: `5432`

##### `postgresql_manage_db`

Data type: `Boolean`

Whether or not to manage the PostgreSQL installation. If false, a database at the specified host and port is expected to exist and the user should have sufficient permissions.

Default value: ``true``

##### `postgresql_db_ssl`

Data type: `Boolean`

Whether to configure SSL connection for PostgresQL database. The configuration is only applied if the PostgresQL database is unmanaged.

Default value: ``false``

##### `postgresql_db_ssl_require`

Data type: `Optional[Boolean]`

Specifies whether pulpcore is configured to require an encrypted connection to the unmanaged PostgreSQL database server.

Default value: ``undef``

##### `postgresql_db_ssl_cert`

Data type: `Optional[Stdlib::Absolutepath]`

Path to the SSL certificate to be used for the SSL connection to PostgreSQL.

Default value: ``undef``

##### `postgresql_db_ssl_key`

Data type: `Optional[Stdlib::Absolutepath]`

Path to the key to be used for the SSL connection to PostgreSQL.

Default value: ``undef``

##### `postgresql_db_ssl_root_ca`

Data type: `Optional[Stdlib::Absolutepath]`

Path to the root certificate authority to validate the certificate supplied by the PostgreSQL database server.

Default value: ``undef``

##### `django_secret_key`

Data type: `String`

SECRET_KEY for Django

Default value: `extlib::cache_data('pulpcore_cache_data', 'secret_key', extlib::random_password(32))`

##### `redis_db`

Data type: `Integer[0]`

Redis DB number to use. By default, Redis supports a DB number of 0 through 15.

Default value: `8`

##### `servername`

Data type: `Stdlib::Fqdn`

Server name of the VirtualHost in the webserver

Default value: `$facts['networking']['fqdn']`

##### `remote_user_environ_name`

Data type: `Optional[String]`

Django remote user environment variable

Default value: ``undef``

##### `allowed_import_path`

Data type: `Array[Stdlib::Absolutepath]`

Allowed paths that pulp can sync from using file:// protocol

Default value: `['/var/lib/pulp/sync_imports']`

##### `worker_count`

Data type: `Integer[0]`

Number of pulpcore workers. Defaults to 8 or the number of CPU cores, whichever is smaller. Enabling more than 8 workers, even with additional CPU cores
available, likely results in performance degradation due to I/O blocking and is not recommended in most cases. Modifying this parameter should
be done incrementally with benchmarking at each step to determine an optimal value for your deployment.

Default value: `min(8, $facts['processors']['count'])`

### `pulpcore::plugin::certguard`

Pulp Certguard plugin

### `pulpcore::plugin::container`

Pulp Container plugin

### `pulpcore::plugin::deb`

Pulp Deb plugin

### `pulpcore::plugin::file`

Pulp File plugin

### `pulpcore::plugin::migration`

Pulp 2 to Pulp 3 Migration Plugin

#### Parameters

The following parameters are available in the `pulpcore::plugin::migration` class.

##### `mongo_db_name`

Data type: `String`

MongoDB database name for Pulp2. Used for migrating Pulp2 content

Default value: `'pulp_database'`

##### `mongo_db_seeds`

Data type: `String`

MongoDB seeds. Used for migrating Pulp2 content

Default value: `'localhost:27017'`

##### `mongo_db_username`

Data type: `Optional[String]`

MongoDB username for Pulp2

Default value: ``undef``

##### `mongo_db_password`

Data type: `Optional[String]`

Password of MongoDB user

Default value: ``undef``

##### `mongo_db_replica_set`

Data type: `Optional[String]`

MongoDB Replica Set

Default value: ``undef``

##### `mongo_db_ssl`

Data type: `Boolean`

Use SSL for MongoDB connection

Default value: ``false``

##### `mongo_db_ssl_keyfile`

Data type: `Optional[Stdlib::Absolutepath]`

SSL Keyfile for MongoDB

Default value: ``undef``

##### `mongo_db_ssl_certfile`

Data type: `Optional[Stdlib::Absolutepath]`

SSL Certfile for MongoDB

Default value: ``undef``

##### `mongo_db_verify_ssl`

Data type: `Boolean`

Whether SSL verification is required for MongoDB

Default value: ``true``

##### `mongo_db_ca_path`

Data type: `Stdlib::Absolutepath`

CA bundle path to use for MongoDB SSL connection

Default value: `'/etc/pki/tls/certs/ca-bundle.crt'`

### `pulpcore::plugin::rpm`

Pulp RPM plugin

## Defined types

### `pulpcore::admin`

Run a pulpcore-manager command

* **See also**
  * exec

#### Parameters

The following parameters are available in the `pulpcore::admin` defined type.

##### `command`

Data type: `String`

The command to run

Default value: `$title`

##### `refreshonly`

Data type: `Boolean`

The command should only be run as a refresh mechanism for when a dependent
object is changed.

Default value: ``false``

##### `unless`

Data type: `Optional[String]`

A test command that checks the state of the target system and restricts
when the exec can run.

Default value: ``undef``

##### `path`

Data type: `Array[Stdlib::Absolutepath]`

The path to look for commands.

Default value: `['/usr/bin']`

##### `user`

Data type: `String`

The user to execute the command.

Default value: `$pulpcore::user`

##### `pulp_settings`

Data type: `Stdlib::Absolutepath`

Root directory for static content

Default value: `$pulpcore::settings_file`

### `pulpcore::plugin`

Install a plugin

#### Parameters

The following parameters are available in the `pulpcore::plugin` defined type.

##### `package_name`

Data type: `String`

The package name to install

Default value: `"python3-pulp-${title}"`

##### `config`

Data type: `Optional[String]`

An optional config in the Pulp settings file

Default value: ``undef``

